@@ -19,7 +19,7 @@
  *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
  *  USA
  *
- *  $Id: channel_mode.h 722 2006-11-22 18:29:54Z stu $
+ *  $Id: channel_mode.h 850 2007-03-05 22:27:07Z tjfontaine $
  */
 
 
@@ -82,7 +82,7 @@
 #define PrivateChannel(x)       (((x)->mode.mode & MODE_PRIVATE))
 #define RegOnlyChannel(x)       ((x) && ((x)->mode.mode & MODE_REGONLY))
 #define SpeakOnlyIfReg(x)       ((x) && ((x)->mode.mode & MODE_SPEAKONLYIFREG))
-#define SSLonlyChannel(x)       ((x) && (x)->mode.mode & MODE_SSLONLY)
+#define SSLonlyChannel(x)       ((x) && ((x)->mode.mode & MODE_SSLONLY))
     
 
 struct ChModeChange
