@@ -19,7 +19,7 @@
  *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
  *  USA
  *
- *  $Id: s_conf.h 828 2007-02-18 20:13:17Z stu $
+ *  $Id: s_conf.h 855 2007-04-09 16:53:20Z stu $
  */
 
 #ifndef INCLUDED_s_conf_h
@@ -542,7 +542,7 @@ extern void yyerror(const char *);
 extern int conf_yy_fatal_error(const char *);
 extern int conf_fbgets(char *, unsigned int, FBFILE *);
 extern void write_conf_line(struct Client *, struct ConfItem *,
-                            const char *, time_t);
+                            const char *, time_t, time_t);
 extern int remove_conf_line(ConfType, struct Client *, const char *,
                             const char *);
 extern void add_temp_line(struct ConfItem *);
