@@ -19,7 +19,7 @@
  *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
  *  USA
  *
- *  $Id: m_kick.c 725 2006-11-22 20:54:21Z stu $
+ *  $Id: m_kick.c 854 2007-04-07 15:58:56Z stu $
  */
 
 #include "stdinc.h"
@@ -63,7 +63,7 @@ _moddeinit(void)
   mod_del_cmd(&kick_msgtab);
 }
 
-const char *_version = "$Revision: 725 $";
+const char *_version = "$Revision: 854 $";
 #endif
 
 /* m_kick()
@@ -156,7 +156,7 @@ m_kick(struct Client *client_p, struct Client *source_p,
         return;
       }
 
-      if (chptr->channelts == 0)
+      if (chptr->channelts == 0 && !IsGod(source_p))
       {
         /* If its a TS 0 channel, do it the old way */
         sendto_one(source_p, form_str(ERR_CHANOPRIVSNEEDED),
