@@ -19,7 +19,7 @@
  *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
  *  USA
  *
- *  $Id: m_nick.c,v 1.1 2002/09/12 02:16:17 cry0gen Exp $
+ *  $Id: m_nick.c 790 2007-02-15 20:43:11Z stu $
  */
 
 #include "stdinc.h"
@@ -33,7 +33,6 @@
 #include "s_conf.h"
 #include "s_stats.h"
 #include "s_user.h"
-#include "hash.h"
 #include "whowas.h"
 #include "s_serv.h"
 #include "send.h"
@@ -46,40 +45,38 @@
 #include "modules.h"
 #include "common.h"
 #include "packet.h"
+#include "irc_getaddrinfo.h"
 
+static void m_nick(struct Client *, struct Client *, int, char **);
+static void mr_nick(struct Client *, struct Client *, int, char **);
+static void ms_nick(struct Client *, struct Client *, int, char **);
+static void ms_uid(struct Client *, struct Client *, int, char **);
 
-static void mr_nick(struct Client *, struct Client *, int, char**);
-static void m_nick(struct Client *, struct Client *, int, char**);
-static void ms_nick(struct Client *, struct Client *, int, char**);
+static void nick_from_server(struct Client *, struct Client *, int, char **,
+                             time_t, char *, char *);
+static void client_from_server(struct Client *, struct Client *, int, char **,
+                               time_t, char *, char *);
+static int check_clean_nick(struct Client *client_p, struct Client *source_p, 
+                            char *nick, char *newnick,
+			    struct Client *server_p);
+static int check_clean_user(struct Client *client_p, char *nick, char *user,
+			    struct Client *server_p);
+static int check_clean_host(struct Client *client_p, char *nick, char *host,
+			    struct Client *server_p);
 
-static void ms_client(struct Client *, struct Client *, int, char**);
-
-static int nick_from_server(struct Client *, struct Client *, int, char **,
-                            time_t, char *);
-static int client_from_server(struct Client *, struct Client *, int, char **,
-                              time_t, char *);
-
-static int check_clean_nick(struct Client *, struct Client *, 
-                            char *, char *, char *);
-static int check_clean_user(struct Client *, char *, char *, char *);
-static int check_clean_host(struct Client *, char *, char *, char *);
-
-static int clean_nick_name(char *);
+static int clean_nick_name(char *, int);
 static int clean_user_name(char *);
 static int clean_host_name(char *);
-
-static int perform_nick_collides(struct Client *, struct Client *,
-                                 struct Client *, int, char **, time_t, char *);
-                            
-
+static void perform_nick_collides(struct Client *, struct Client *, struct Client *,
+				  int, char **, time_t, char *, char *, char *);
 struct Message nick_msgtab = {
   "NICK", 0, 0, 1, 0, MFLG_SLOW, 0,
-  {mr_nick, m_nick, ms_nick, m_nick}
+  {mr_nick, m_nick, ms_nick, m_ignore, m_nick, m_ignore}
 };
 
-struct Message client_msgtab = {
-  "CLIENT", 0, 0, 10, 0, MFLG_SLOW, 0,
-  {m_ignore, m_ignore, ms_client, m_ignore}
+struct Message uid_msgtab = {
+  "UID", 0, 0, 10, 0, MFLG_SLOW, 0,
+  {m_ignore, m_ignore, ms_uid, m_ignore, m_ignore, m_ignore}
 };
 
 #ifndef STATIC_MODULES
@@ -87,73 +84,75 @@ void
 _modinit(void)
 {
   mod_add_cmd(&nick_msgtab);
-  mod_add_cmd(&client_msgtab);
+  mod_add_cmd(&uid_msgtab);
 }
 
 void
 _moddeinit(void)
 {
   mod_del_cmd(&nick_msgtab);
-  mod_del_cmd(&client_msgtab);
+  mod_del_cmd(&uid_msgtab);
 }
 
-const char *_version = "$Revision: 1.1 $";
+const char *_version = "$Revision: 790 $";
 #endif
 
-/*
- * mr_nick()
+/* mr_nick()
  *
  *       parv[0] = sender prefix
  *       parv[1] = nickname
  */
-static void mr_nick(struct Client *client_p, struct Client *source_p, 
-                    int parc, char *parv[])
+static void
+mr_nick(struct Client *client_p, struct Client *source_p,
+        int parc, char *parv[])
 {
-  struct   Client *target_p, *uclient_p;
-  char     nick[NICKLEN];
-  char*    s;
+  struct Client *target_p, *uclient_p;
+  char nick[NICKLEN];
+  char *s;
   dlink_node *ptr;
    
-  if(parc < 2 || BadPtr(parv[1]))
+  if (parc < 2 || EmptyString(parv[1]))
   {
     sendto_one(source_p, form_str(ERR_NONICKNAMEGIVEN),
-               me.name, BadPtr(parv[0]) ? "*" : parv[0]);
+               me.name, EmptyString(parv[0]) ? "*" : parv[0]);
     return;
   }
 
   /* Terminate the nick at the first ~ */
-  if ((s = strchr(parv[1], '~')))
+  /* XXX - Is this still needed??      */
+  if ((s = strchr(parv[1], '~')) != NULL)
     *s = '\0';
 
   /* copy the nick and terminate it */
-  strlcpy(nick, parv[1], NICKLEN);
+  strlcpy(nick, parv[1], sizeof(nick));
 
   /* check the nickname is ok */
-  if(!clean_nick_name(nick))
+  if (!clean_nick_name(nick, 1))
   {
     sendto_one(source_p, form_str(ERR_ERRONEUSNICKNAME),
-               me.name, BadPtr(parv[0]) ? "*" : parv[0], parv[1]);
+               me.name, EmptyString(parv[0]) ? "*" : parv[0], parv[1]);
     return;
   }
 
   /* check if the nick is resv'd */
-  if(find_nick_resv(nick))
+  if (find_matching_name_conf(NRESV_TYPE, nick, NULL, NULL, 0) &&
+      !IsExemptResv(source_p))
   {
-    sendto_one(source_p, form_str(ERR_UNAVAILRESOURCE),
-               me.name, BadPtr(parv[0]) ? "*" : parv[0], nick);
+    sendto_one(source_p, form_str(ERR_ERRONEUSNICKNAME),
+               me.name, EmptyString(parv[0]) ? "*" : parv[0], nick);
     return;
   }
 
   if ((target_p = find_client(nick)) == NULL)
   {
-    if(!ServerInfo.hub && uplink && IsCapable(uplink, CAP_LL))
+    if (!ServerInfo.hub && uplink && IsCapable(uplink, CAP_LL))
     {
       /* We don't know anyone called nick, but our hub might */
-      for(ptr = unknown_list.head; ptr; ptr = ptr->next)
+      DLINK_FOREACH(ptr, unknown_list.head)
       {
         uclient_p = ptr->data;
 
-	if(!strcmp(nick, uclient_p->llname))
+	if (!strcmp(nick, uclient_p->llname))
 	{
 	
 	  /* We're already waiting for a reply about this nick
@@ -180,7 +179,7 @@ static void mr_nick(struct Client *client_p, struct Client *source_p,
       return;
     }
   }
-  else if(source_p == target_p)
+  else if (source_p == target_p)
   {
     strcpy(source_p->name, nick);
     return;
@@ -191,19 +190,19 @@ static void mr_nick(struct Client *client_p, struct Client *source_p,
   }
 }
 
-/*
- * m_nick()
+/* m_nick()
  *
  *     parv[0] = sender prefix
  *     parv[1] = nickname
  */
- static void m_nick(struct Client *client_p, struct Client *source_p,
-                   int parc, char *parv[])
+static void
+m_nick(struct Client *client_p, struct Client *source_p,
+       int parc, char *parv[])
 {
-  char     nick[NICKLEN];
-  struct   Client *target_p;
+  char nick[NICKLEN];
+  struct Client *target_p;
 
-  if(parc < 2 || BadPtr(parv[1]))
+  if (parc < 2 || EmptyString(parv[1]))
   {
     sendto_one(source_p, form_str(ERR_NONICKNAMEGIVEN),
                me.name, parv[0]);
@@ -211,23 +210,25 @@ static void mr_nick(struct Client *client_p, struct Client *source_p,
   }
 
   /* mark end of grace period, to prevent nickflooding */
-  if(!IsFloodDone(source_p))
+  if (!IsFloodDone(source_p))
     flood_endgrace(source_p);
 
   /* terminate nick to NICKLEN */
-  strlcpy(nick, parv[1], NICKLEN);
+  strlcpy(nick, parv[1], sizeof(nick));
 
   /* check the nickname is ok */
-  if(!clean_nick_name(nick))
+  if (!clean_nick_name(nick, 1))
   {
     sendto_one(source_p, form_str(ERR_ERRONEUSNICKNAME),
                me.name, parv[0], nick);
     return;
   }
 
-  if(find_nick_resv(nick))
+  if (find_matching_name_conf(NRESV_TYPE, nick,
+			     NULL, NULL, 0) && !IsExemptResv(source_p) &&
+     !(IsOper(source_p) && ConfigFileEntry.oper_pass_resv))
   {
-    sendto_one(source_p, form_str(ERR_UNAVAILRESOURCE),
+    sendto_one(source_p, form_str(ERR_ERRONEUSNICKNAME),
                me.name, parv[0], nick);
     return;
   }
@@ -238,33 +239,25 @@ static void mr_nick(struct Client *client_p, struct Client *source_p,
      * equivalent nicknames ie: [nick] -> {nick}
      */
 
-    if(target_p == source_p)
+    if (target_p == source_p)
     {
       /* check the nick isnt exactly the same */
-      if(strcmp(target_p->name, nick))
-      {
-        change_local_nick(client_p, source_p, nick);
-	return;
-      }
-      else
-      {
-        /* client is doing :old NICK old
-	 * ignore it..
-	 */
-        return;
-      }
+      if (!strcmp(target_p->name, nick))
+        return; /* client is doing :old NICK old ignore it. */
+
+      change_local_nick(client_p, source_p, nick);
+      return;
     }
 
-    /* if the client that has the nick isnt registered yet (nick but no
+    /* if the client that has the nick isn't registered yet (nick but no
      * user) then drop the unregged client
      */
-    if(IsUnknown(target_p))
+    if (IsUnknown(target_p))
     {
       /* the old code had an if(MyConnect(target_p)) here.. but I cant see
        * how that can happen, m_nick() is local only --fl_
        */
-      
-      exit_client(NULL, target_p, &me, "Overridden");
+      exit_client(target_p, &me, "Overridden");
       change_local_nick(client_p, source_p, nick);
       return;
     }
@@ -274,14 +267,14 @@ static void mr_nick(struct Client *client_p, struct Client *source_p,
                  parv[0], nick);
       return;
     }
-
   }
   else
   {
-    if(!ServerInfo.hub && uplink && IsCapable(uplink, CAP_LL))
+    if (!ServerInfo.hub && uplink && IsCapable(uplink, CAP_LL))
     {
       /* The uplink might know someone by this name already. */
-      sendto_one(uplink, ":%s NBURST %s %s %s", me.name, nick,
+      sendto_one(uplink, ":%s NBURST %s %s %s",
+                 ID_or_name(&me, uplink), nick,
                  nick, source_p->name);
       return;
     }
@@ -312,92 +305,94 @@ static void mr_nick(struct Client *client_p, struct Client *source_p,
  *    parv[7] = server
  *    parv[8] = ircname
  */
-static void ms_nick(struct Client *client_p, struct Client *source_p,
-                    int parc, char *parv[])
+static void
+ms_nick(struct Client *client_p, struct Client *source_p,
+        int parc, char *parv[])
 {
   struct Client* target_p;
-  char     nick[NICKLEN];
-  time_t   newts = 0;
+  char nick[NICKLEN];
+  char ngecos[REALLEN + 1];
+  time_t newts = 0;
+  char *nnick = parv[1];
+  char *nhop = parv[2];
+  char *nts = parv[3];
+  char *nusername = parv[5];
+  char *nhost = parv[6];
+  char *nserver = parv[7];
 
-  /* XXX BadPtr is needed */
-  if(parc < 2 || BadPtr(parv[1]))
-  {
-    sendto_one(source_p, form_str(ERR_NONICKNAMEGIVEN), me.name, parv[0]);
+  if (parc < 2 || EmptyString(nnick))
     return;
-  }
 
-  /* parc == 3 on nickchange, parc == 9 on new nick */
-  if((IsClient(source_p) && (parc != 3)) || (IsServer(source_p) && (parc != 9)))
-  {
-    char tbuf[BUFSIZE] = { 0 };
-    int j;
-
-    for(j = 0; j < parc; j++)
-    {
-      strcat(tbuf, parv[j]);
-      strcat(tbuf, " ");
-    }
-
-    sendto_gnotice_flags(FLAGS_ALL, L_ALL, me.name, &me, NULL,
-                         "Dropping server %s due to (invalid) command 'NICK' "
-                         "with only %d arguments.  (Buf: '%s')",
-                         client_p->name, parc, tbuf);
-    ilog(L_CRIT, "Insufficient parameters (%d) for command 'NICK' from %s.  Buf: %s",
-         parc, client_p->name, tbuf);
-    exit_client(client_p, client_p, client_p, "Not enough arguments to server command.");
-    return;
-  }
-
-  /* fix the length of the nick */
-  strlcpy(nick, parv[1], NICKLEN);
-
-  if(check_clean_nick(client_p, source_p, nick, parv[1], parv[7]))
-    return;
+  /* fix the lengths */
+  strlcpy(nick, nnick, sizeof(nick));
 
   if (parc == 9)
-    {
-      if (check_clean_user(client_p, nick, parv[5], parv[7]) ||
-          check_clean_host(client_p, nick, parv[6], parv[7]))
-        return;
+  {
+    struct Client *server_p = find_server(nserver);
 
-      /* check the length of the clients gecos */
-      if(strlen(parv[8]) > REALLEN)
-        {
-          sendto_gnotice_flags(FLAGS_ALL, L_OPER, me.name, &me, NULL, "Long realname from server %s for %s",
-                         parv[7], parv[1]);
-          parv[8][REALLEN] = '\0';
-        }
+    strlcpy(ngecos, parv[8], sizeof(ngecos));
 
-      if (IsServer(source_p))
-        newts = atol(parv[3]);
-    }
-  else
+    if (server_p == NULL)
     {
-      if (!IsServer(source_p))
-        newts = atol(parv[2]);
+      sendto_gnotice_flags(UMODE_ALL, L_ALL, me.name, &me, NULL,
+			   "Invalid server %s from %s for NICK %s",
+			   nserver, source_p->name, nick);
+      sendto_one(client_p, ":%s KILL %s :%s (Server doesn't exist!)",
+                 me.name, nick, me.name);
+      return;
     }
 
+    if (check_clean_nick(client_p, source_p, nick, nnick, server_p) ||
+        check_clean_user(client_p, nick, nusername, server_p) ||
+	check_clean_host(client_p, nick, nhost, server_p))
+      return;
+
+    /* check the length of the clients gecos */
+    if (strlen(parv[8]) > REALLEN)
+      sendto_gnotice_flags(UMODE_ALL, L_ALL, me.name, &me, NULL,
+			   "Long realname from server %s for %s",
+			   nserver, nnick);
+
+    if (IsServer(source_p))
+      newts = atol(nts);
+  }
+  else if (parc == 3)
+  {
+    if (IsServer(source_p))
+      /* Server's cant change nicks.. */
+      return;
+
+    if (check_clean_nick(client_p, source_p, nick, nnick,
+			 source_p->servptr))
+      return;
+    
+    /*
+     * Yes, this is right. HOP field is the TS field for parc = 3
+     */ 
+    newts = atol(nhop);	
+  }
+
   /* if the nick doesnt exist, allow it and process like normal */
   if (!(target_p = find_client(nick)))
   {
-    nick_from_server(client_p,source_p,parc,parv,newts,nick);
+    nick_from_server(client_p, source_p, parc, parv, newts, nick, ngecos);
     return;
   }
 
   /* we're not living in the past anymore, an unknown client is local only. */
-  if(IsUnknown(target_p))
+  if (IsUnknown(target_p))
   {
-    exit_client(NULL, target_p, &me, "Overridden");
-    nick_from_server(client_p,source_p,parc,parv,newts,nick);
+    exit_client(target_p, &me, "Overridden");
+    nick_from_server(client_p, source_p, parc, parv, newts, nick, ngecos);
     return;
   }
 
-  if(target_p == source_p)
+  if (target_p == source_p)
   {
-    if(strcmp(target_p->name, nick))
+    if (strcmp(target_p->name, nick))
     {
       /* client changing case of nick */
-      nick_from_server(client_p,source_p,parc,parv,newts,nick);
+      nick_from_server(client_p, source_p, parc, parv, newts, nick, ngecos);
       return;
     }
     else
@@ -406,66 +401,66 @@ static void ms_nick(struct Client *client_p, struct Client *source_p,
   }
 
   perform_nick_collides(source_p, client_p, target_p,
-                        parc, parv, newts, nick);
-			
-
+                        parc, parv, newts, nick, ngecos, NULL);
 }
 
-/*
- * ms_client()
+/* ms_uid()
+ *
+ *  parv[0] = sender prefix
+ *  parv[1] = nickname
+ *  parv[2] = hop count
+ *  parv[3] = TS
+ *  parv[4] = umode
+ *  parv[5] = username
+ *  parv[6] = hostname
+ *  parv[7] = ip
+ *  parv[8] = uid
+ *  parv[9] = ircname (gecos)
  */
 static void
-ms_client(struct Client *client_p, struct Client *source_p,
-	  int parc, char *parv[])
+ms_uid(struct Client *client_p, struct Client *source_p,
+       int parc, char *parv[])
 {
-  struct Client* target_p;
-  char     nick[NICKLEN];
-  time_t   newts = 0;
-  char    *id;
-  char    *name;
+  struct Client *target_p;
+  char nick[NICKLEN];
+  char ugecos[REALLEN + 1];
+  time_t newts = 0;
+  char *unick = parv[1];
+  char *uts = parv[3];
+  char *uname = parv[5];
+  char *uhost = parv[6];
+  char *uid = parv[8];
 
-  id = parv[8];
-  name = parv[9];
-
-  /* XXX can this happen ? */
-  if (BadPtr(parv[1]))
+  if (EmptyString(unick))
     return;
 
-  /* XXX can this happen ? */
-  if (BadPtr(parv[1]))
+  /* Fix the lengths */
+  strlcpy(nick, parv[1], sizeof(nick));
+  strlcpy(ugecos, parv[9], sizeof(ugecos));
+
+  if (check_clean_nick(client_p, source_p, nick, unick, source_p) ||
+      check_clean_user(client_p, nick, uname, source_p) ||
+      check_clean_host(client_p, nick, uhost, source_p))
     return;
 
-  /* parse the nickname */
-  strlcpy(nick, parv[1], NICKLEN);
-
-  /* check the nicknames, usernames and hostnames are ok */
-  if(check_clean_nick(client_p, source_p, nick, parv[1], parv[7]) ||
-     check_clean_user(client_p, nick, parv[5], parv[7]) ||
-     check_clean_host(client_p, nick, parv[6], parv[7]))
-    return;
-
-  /* check length of clients gecos */
-  if (strlen(name) > REALLEN)
-  {
-    sendto_gnotice_flags(FLAGS_ALL, L_OPER, me.name, &me, NULL, "Long realname from server %s for %s",
+  if (strlen(parv[9]) > REALLEN)
+    sendto_realops_flags(UMODE_ALL, L_ALL, "Long realname from server %s for %s",
                          parv[0], parv[1]);
-    name[REALLEN] = '\0';			 
-  }
 
-  newts = atol(parv[3]);
- 
+  newts = atol(uts);
+
   /* if there is an ID collision, kill our client, and kill theirs.
    * this may generate 401's, but it ensures that both clients always
    * go, even if the other server refuses to do the right thing.
    */
-  if((target_p = find_id(id)))
+  if ((target_p = hash_find_id(uid)) != NULL)
   {
-    sendto_gnotice_flags(FLAGS_ALL, L_OPER, me.name, &me, NULL,
+    sendto_realops_flags(UMODE_ALL, L_ALL,
 		         "ID collision on %s(%s <- %s)(both killed)",
 			 target_p->name, target_p->from->name,
 			 client_p->name);
 
-    if(ServerInfo.hub && IsCapable(client_p, CAP_LL))
+    if (ServerInfo.hub && IsCapable(client_p, CAP_LL))
       add_lazylinkclient(client_p, source_p);
 
     kill_client_ll_serv_butone(NULL, target_p, "%s (ID collision)",
@@ -473,72 +468,66 @@ ms_client(struct Client *client_p, struct Client *source_p,
 
     ServerStats->is_kill++;
 	    
-    target_p->flags |= FLAGS_KILLED;
-    exit_client(client_p, target_p, &me, "ID Collision");
+    SetKilled(target_p);
+    exit_client(target_p, &me, "ID Collision");
     return;
   }
     
-  if (!(target_p = find_client(nick)))
+  if ((target_p = find_client(unick)) == NULL)
+    client_from_server(client_p, source_p, parc, parv, newts, nick, ugecos);
+  else if (IsUnknown(target_p))
   {
-    client_from_server(client_p,source_p,parc,parv,newts,nick);
-    return;
+    exit_client(target_p, &me, "Overridden");
+    client_from_server(client_p, source_p, parc, parv, newts, nick, ugecos);
   }
-
-
-  if(IsUnknown(target_p))
-  {
-    exit_client(NULL, target_p, &me, "Overridden");
-    client_from_server(client_p,source_p,parc,parv,newts,nick);
-    return;
-  }
-
-  perform_nick_collides(source_p, client_p, target_p,
-                        parc, parv, newts, nick);
+  else
+    perform_nick_collides(source_p, client_p, target_p,
+                          parc, parv, newts, nick, ugecos, uid);
 }			  
 
-
-/* 
- * check_clean_nick()
- * 
+/* check_clean_nick()
+ *
  * input	- pointer to source
+ *		- 
  *		- nickname
  *		- truncated nickname
  *		- origin of client
+ *		- pointer to server nick is coming from
  * output	- none
  * side effects - if nickname is erroneous, or a different length to
  *                truncated nickname, return 1
  */
-static int check_clean_nick(struct Client *client_p, struct Client *source_p, 
-                            char *nick, char *newnick, char *server)
+static int
+check_clean_nick(struct Client *client_p, struct Client *source_p, 
+                 char *nick, char *newnick, struct Client *server_p)
 {
   /* the old code did some wacky stuff here, if the nick is invalid, kill it
    * and dont bother messing at all
    */
-
-  if(!clean_nick_name(nick) || strcmp(nick, newnick))
+  if (!clean_nick_name(nick, 0) || strcmp(nick, newnick))
   {
     ServerStats->is_kill++;
-    sendto_gnotice_flags(FLAGS_DEBUG, L_OPER, me.name, &me, NULL,
+    sendto_gnotice_flags(UMODE_DEBUG, L_ALL, me.name, &me, NULL,
                          "Bad Nick: %s From: %s(via %s)",
-                         nick, server, client_p->name);
+                         nick, server_p->name, client_p->name);
 
     sendto_one(client_p, ":%s KILL %s :%s (Bad Nickname)",
                me.name, newnick, me.name);
 
     /* bad nick change */
-    if(source_p != client_p)
+    if (source_p != client_p)
     {
       kill_client_ll_serv_butone(client_p, source_p,
                                  "%s (Bad Nickname)",
 				 me.name);
-      source_p->flags |= FLAGS_KILLED;
-      exit_client(client_p, source_p, &me, "Bad Nickname");
+      SetKilled(source_p);
+      exit_client(source_p, &me, "Bad Nickname");
     }
 
-    return 1;
+    return (1);
   }
 
-  return 0;
+  return (0);
 }
 
 /* check_clean_user()
@@ -552,27 +541,27 @@ static int check_clean_nick(struct Client *client_p, struct Client *source_p,
  */
 static int
 check_clean_user(struct Client *client_p, char *nick, 
-		 char *user, char *server)
+                 char *user, struct Client *server_p)
 {
-  if(strlen(user) > USERLEN)
+  if (strlen(user) > USERLEN)
   {
     ServerStats->is_kill++;
-    sendto_gnotice_flags(FLAGS_DEBUG, L_OPER, me.name, &me, NULL,
+    sendto_gnotice_flags(UMODE_DEBUG, L_ALL, me.name, &me, NULL,
                          "Long Username: %s Nickname: %s From: %s(via %s)",
-			 user, nick, server, client_p->name);
+			 user, nick, server_p->name, client_p->name);
 
     sendto_one(client_p, ":%s KILL %s :%s (Bad Username)",
                me.name, nick, me.name);
   
-    return 1;
+    return (1);
   }
 
-  if(!clean_user_name(user))
-    sendto_gnotice_flags(FLAGS_DEBUG, L_OPER, me.name, &me, NULL,
+  if (!clean_user_name(user))
+    sendto_gnotice_flags(UMODE_DEBUG, L_ALL, me.name, &me, NULL,
                          "Bad Username: %s Nickname: %s From: %s(via %s)",
-			 user, nick, server, client_p->name);
+			 user, nick, server_p->name, client_p->name);
 			 
-  return 0;
+  return (0);
 }
 
 /* check_clean_host()
@@ -586,53 +575,56 @@ check_clean_user(struct Client *client_p, char *nick,
  */
 static int
 check_clean_host(struct Client *client_p, char *nick,
-		 char *host, char *server)
+                 char *host, struct Client *server_p)
 {
-  if(strlen(host) > HOSTLEN)
+  if (strlen(host) > HOSTLEN)
   {
     ServerStats->is_kill++;
-    sendto_gnotice_flags(FLAGS_DEBUG, L_OPER, me.name, &me, NULL,
+    sendto_gnotice_flags(UMODE_DEBUG, L_ALL, me.name, &me, NULL,
                          "Long Hostname: %s Nickname: %s From: %s(via %s)",
-			 host, nick, server, client_p->name);
+			 host, nick, server_p->name, client_p->name);
 
     sendto_one(client_p, ":%s KILL %s :%s (Bad Hostname)",
                me.name, nick, me.name);
 
-    return 1;
+    return (1);
   }
 
-  if(!clean_host_name(host))
-    sendto_gnotice_flags(FLAGS_DEBUG, L_OPER, me.name, &me, NULL,
+  if (!clean_host_name(host))
+    sendto_gnotice_flags(UMODE_DEBUG, L_ALL, me.name, &me, NULL,
                          "Bad Hostname: %s Nickname: %s From: %s(via %s)",
-			 host, nick, server, client_p->name);
+			 host, nick, server_p->name, client_p->name);
 
-  return 0;
+  return (0);
 }
 
 /* clean_nick_name()
  *
  * input	- nickname
+ *              - whether it's a local nick (1) or remote (0)
  * output	- none
  * side effects - walks through the nickname, returning 0 if erroneous
  */
 static int
-clean_nick_name(char *nick)
+clean_nick_name(char *nick, int local)
 {
   assert(nick);
-  if(nick == NULL)
-    return 0;
+  if (nick == NULL)
+    return (0);
 
-  /* nicks cant start with a digit or - */
-  if (*nick == '-' || IsDigit(*nick))
-    return 0;
+  /* nicks cant start with a digit or - or be 0 length */
+  /* This closer duplicates behaviour of hybrid-6 */
+
+  if (*nick == '-' || (IsDigit(*nick) && local) || *nick == '\0')
+    return (0);
 
   for(; *nick; nick++)
   {
-    if(!IsNickChar(*nick))
-      return 0;
+    if (!IsNickChar(*nick))
+      return (0);
   }
 
-  return 1;
+  return (1);
 }
 
 /* clean_user_name()
@@ -645,14 +637,13 @@ static int
 clean_user_name(char *user)
 {
   assert(user);
-  if(user == NULL)
+  if (user == NULL)
     return 0;
     
   for(; *user; user++)
   {
-    if(!IsUserChar(*user))
+    if (!IsUserChar(*user))
       return 0;
-    
   }
 
   return 1;
@@ -667,38 +658,37 @@ static int
 clean_host_name(char *host)
 {
   assert(host);
-  if(host == NULL)
+  if (host == NULL)
     return 0;
   for(; *host; host++)
   {
-    if(!IsHostChar(*host))
+    if (!IsHostChar(*host))
       return 0;
   }
 
   return 1;
 }
 
-
 /*
  * nick_from_server()
  */
-static int
+static void
 nick_from_server(struct Client *client_p, struct Client *source_p, int parc,
-                 char *parv[], time_t newts,char *nick)
+                 char *parv[], time_t newts, char *nick, char *ngecos)
 {
-  if(IsServer(source_p))
+  if (IsServer(source_p))
   {
     /* A server introducing a new client, change source */
     source_p = make_client(client_p);
-    add_client_to_list(source_p);
+    dlinkAdd(source_p, &source_p->node, &global_client_list);
 
     /* We don't need to introduce leafs clients back to them! */
-    if (ConfigFileEntry.hub && IsCapable(client_p, CAP_LL))
+    if (ServerInfo.hub && IsCapable(client_p, CAP_LL))
       add_lazylinkclient(client_p, source_p);
 
-    if(parc > 2)
+    if (parc > 2)
       source_p->hopcount = atoi(parv[2]);
-    if(newts)
+    if (newts)
       source_p->tsinfo = newts;
     else
     {
@@ -707,136 +697,151 @@ nick_from_server(struct Client *client_p, struct Client *source_p, int parc,
     }
 
     /* copy the nick in place */
-    (void)strcpy(source_p->name, nick);
-    (void)add_to_client_hash_table(nick, source_p);
+    strcpy(source_p->name, nick);
+    hash_add_client(source_p);
 
     if (parc > 8)
     {
-       int   flag;
-       char *m;
+      unsigned int flag;
+      char *m;
 
-       /* parse usermodes */
-       m = &parv[4][1];
-       while (*m)
-       {
-         flag = user_modes_from_c_to_bitmask[(unsigned char)*m];
-	 if(!(source_p->umodes & FLAGS_INVISIBLE) && (flag & FLAGS_INVISIBLE))
-	   Count.invisi++;
-	 if(!(source_p->umodes & FLAGS_OPER) && (flag & FLAGS_OPER))
-	   Count.oper++;
+      /* parse usermodes */
+      m = &parv[4][1];
 
-	 source_p->umodes |= flag & SEND_UMODES;
-	 m++;
+      while (*m)
+      {
+        flag = user_modes[(unsigned char)*m];
+        if (!(source_p->umodes & UMODE_INVISIBLE) && (flag & UMODE_INVISIBLE))
+	  Count.invisi++;
+        if (!(source_p->umodes & UMODE_OPER) && (flag & UMODE_OPER))
+	  Count.oper++;
+
+        source_p->umodes |= flag & SEND_UMODES;
+        m++;
       }
 
-      return do_remote_user(nick, client_p, source_p, parv[5], parv[6],
-                            parv[7], parv[8], NULL);
+      register_remote_user(client_p, source_p, parv[5], parv[6],
+                           parv[7], ngecos);
+      return;
     }
   }
-  else if(source_p->name[0])
+  else if (source_p->name[0])
   {
     /* client changing their nick */
-    if(irccmp(parv[0], nick))
-      source_p->tsinfo = newts ? newts : CurrentTime;
-
-    sendto_common_channels_local(source_p, ":%s!%s@%s NICK :%s",
-                                 source_p->name,source_p->username,source_p->host,
-                                 nick);
-
-    if (source_p->user)
+    if (irccmp(parv[0], nick))
     {
-      add_history(source_p,1);
-      sendto_server(client_p, source_p, NULL, NOCAPS, NOCAPS, NOFLAGS,
-                    ":%s NICK %s :%lu",
-		      parv[0], nick, (unsigned long) source_p->tsinfo);
+      del_all_accepts(source_p);
+      source_p->tsinfo = newts ? newts : CurrentTime;
     }
+
+    sendto_common_channels_local(source_p, 1, ":%s!%s@%s NICK :%s",
+                                 source_p->name,source_p->username,
+                                 source_p->host, nick);
+
+    add_history(source_p, 1);
+    sendto_server(client_p, source_p, NULL, CAP_TS6, NOCAPS, NOFLAGS,
+                  ":%s NICK %s :%lu",
+                  ID(source_p), nick, (unsigned long)source_p->tsinfo);
+    sendto_server(client_p, source_p, NULL, NOCAPS, CAP_TS6, NOFLAGS,
+                  ":%s NICK %s :%lu",
+                  parv[0], nick, (unsigned long)source_p->tsinfo);
   }
 
   /* set the new nick name */
-  if(source_p->name[0])
-    del_from_client_hash_table(source_p->name, source_p);
+  if (source_p->name[0])
+    hash_del_client(source_p);
 
   strcpy(source_p->name, nick);
-  add_to_client_hash_table(nick, source_p);
-
-  /* remove all accepts pointing to the client */
-  del_all_accepts(source_p);
-
-  return 0;
+  hash_add_client(source_p);
 }
 
-
 /*
  * client_from_server()
  */
-static int
+static void
 client_from_server(struct Client *client_p, struct Client *source_p, int parc,
-                   char *parv[], time_t newts,char *nick)
+                   char *parv[], time_t newts, char *nick, char *ugecos)
 {
-  char *name;
-  char *id;
-  int flag;
   char *m;
-
-  id = parv[8];
-  name = parv[9];
+  unsigned int flag;
+  const char *servername = source_p->name;
+  struct addrinfo hints, *res;
 
   source_p = make_client(client_p);
-  add_client_to_list(source_p);
+  dlinkAdd(source_p, &source_p->node, &global_client_list);
 
   /* We don't need to introduce leafs clients back to them! */
-  if (ConfigFileEntry.hub && IsCapable(client_p, CAP_LL))
+  if (ServerInfo.hub && IsCapable(client_p, CAP_LL))
     add_lazylinkclient(client_p, source_p);
 
   source_p->hopcount = atoi(parv[2]);
   source_p->tsinfo = newts;
 
   /* copy the nick in place */
-  (void)strcpy(source_p->name, nick);
-  (void)add_to_client_hash_table(nick, source_p);
-  add_to_id_hash_table(id, source_p);
+  strcpy(source_p->name, nick);
+  strlcpy(source_p->id, parv[8], sizeof(source_p->id));
+  strlcpy(source_p->sockhost, parv[7], sizeof(source_p->sockhost));
+
+  memset(&hints, 0, sizeof(hints));
+  hints.ai_family = AF_UNSPEC;
+  hints.ai_flags = AI_PASSIVE | AI_NUMERICHOST;
+
+  irc_getaddrinfo(parv[7], 0, &hints, &res);
+
+  if(res != NULL)
+  {
+    memcpy(&source_p->ip, res->ai_addr, res->ai_addrlen);
+    source_p->ip.ss_len = res->ai_addrlen;
+    irc_freeaddrinfo(res);
+  }
+
+  hash_add_client(source_p);
+  hash_add_id(source_p);
 
   /* parse usermodes */
   m = &parv[4][1];
   while (*m)
   {
-    flag = user_modes_from_c_to_bitmask[(unsigned char)*m];
-    if(flag & FLAGS_INVISIBLE)
+    flag = user_modes[(unsigned char)*m];
+    if (flag & UMODE_INVISIBLE)
       Count.invisi++;
-    if(flag & FLAGS_OPER)
+    if (flag & UMODE_OPER)
       Count.oper++;
 
     source_p->umodes |= flag & SEND_UMODES;
     m++;
-
   }
 
-  return do_remote_user(nick, client_p, source_p, parv[5], parv[6],
-                        parv[7], name, id);
-}			
-  
-static int 
+  register_remote_user(client_p, source_p, parv[5], parv[6],
+                       servername, ugecos);
+}
+
+static void
 perform_nick_collides(struct Client *source_p, struct Client *client_p, 
                       struct Client *target_p, int parc, char *parv[], 
-		      time_t newts, char *nick)
+                      time_t newts, char *nick, char *gecos, char *uid)
 {
   int sameuser;
   
   /* server introducing new nick */
-  if(IsServer(source_p))
+  if (IsServer(source_p))
   {
     /* if we dont have a ts, or their TS's are the same, kill both */
-    if(!newts || !target_p->tsinfo ||
-       (newts == target_p->tsinfo))
+    if (!newts || !target_p->tsinfo || (newts == target_p->tsinfo))
     {
-      sendto_gnotice_flags(FLAGS_ALL, L_OPER, me.name, &me, NULL,
+      sendto_gnotice_flags(UMODE_ALL, L_ALL, me.name, &me, NULL,
                            "Nick collision on %s(%s <- %s)(both killed)",
 			   target_p->name, target_p->from->name,
 			   client_p->name);
       
-      if(ServerInfo.hub && IsCapable(client_p,CAP_LL))
+      if (ServerInfo.hub && IsCapable(client_p,CAP_LL))
         add_lazylinkclient(client_p, target_p);
 
+      /* if we have a UID, issue a kill for it */
+      if (uid)
+        sendto_one(client_p, ":%s KILL %s :%s (Nick collision (new))",
+                   me.id, uid, me.name);
+
       kill_client_ll_serv_butone(NULL, target_p,
                                  "%s (Nick collision (new))",
 				 me.name);
@@ -844,15 +849,15 @@ perform_nick_collides(struct Client *source_p, struct Client *client_p,
       sendto_one(target_p, form_str(ERR_NICKCOLLISION),
                  me.name, target_p->name, target_p->name);
 
-      target_p->flags |= FLAGS_KILLED;
-      exit_client(client_p, target_p, &me, "Nick collision (new)");
-      return 0;
+      SetKilled(target_p);
+      exit_client(target_p, &me, "Nick collision (new)");
+      return;
     }
     /* the timestamps are different */
     else
     {
-      sameuser = (target_p->user) && !irccmp(target_p->username, parv[5])
-                 && !irccmp(target_p->host, parv[6]);
+      sameuser = !irccmp(target_p->username, parv[5]) &&
+                 !irccmp(target_p->host, parv[6]);
    
       /* if the users are the same (loaded a client on a different server)
        * and the new users ts is older, or the users are different and the
@@ -861,18 +866,22 @@ perform_nick_collides(struct Client *source_p, struct Client *client_p,
       if ((sameuser && newts < target_p->tsinfo) ||
          (!sameuser && newts > target_p->tsinfo))
       {
+        if (uid)
+          sendto_one(client_p, ":%s KILL %s :%s (Nick collision (new))",
+                     me.id, uid, me.name);
+
         client_burst_if_needed(client_p, target_p);
-	return 0;
+	return;
       }
       else
       {
-        if(sameuser)
-	  sendto_gnotice_flags(FLAGS_ALL, L_OPER, me.name, &me, NULL,
+        if (sameuser)
+	  sendto_realops_flags(UMODE_ALL, L_ALL,
 	                  "Nick collision on %s(%s <- %s)(older killed)",
 			  target_p->name, target_p->from->name,
 			  client_p->name);
         else
-	  sendto_gnotice_flags(FLAGS_ALL, L_OPER, me.name, &me, NULL,
+	  sendto_realops_flags(UMODE_ALL, L_ALL,
 	                  "Nick collision on %s(%s <- %s)(newer killed)",
 			  target_p->name, target_p->from->name,
 			  client_p->name);
@@ -888,24 +897,23 @@ perform_nick_collides(struct Client *source_p, struct Client *client_p,
 	                           "%s (Nick collision (new))",
 				   me.name);
 
-        target_p->flags |= FLAGS_KILLED;
-	(void)exit_client(client_p, target_p, &me, "Nick collision");
+        SetKilled(target_p);
+	exit_client(target_p, &me, "Nick collision");
 	
-	if(parc == 9)
-	  nick_from_server(client_p,source_p,parc,parv,newts,nick);
-	else if(parc == 10)
-	  client_from_server(client_p,source_p,parc,parv,newts,nick);
+	if (parc == 9)
+	  nick_from_server(client_p, source_p, parc, parv, newts, nick, gecos);
+	else if (parc == 10)
+	  client_from_server(client_p, source_p, parc, parv, newts, nick, gecos);
 	  
-	return 0;
+	return;
       }
     }
   }
 
   /* its a client changing nick and causing a collide */
-  if(!newts || !target_p->tsinfo || (newts == target_p->tsinfo) ||
-       !source_p->user)
+  if (!newts || !target_p->tsinfo || (newts == target_p->tsinfo))
     {
-      sendto_gnotice_flags(FLAGS_ALL, L_OPER, me.name, &me, NULL,
+      sendto_gnotice_flags(UMODE_ALL, L_ALL, me.name, &me, NULL,
                  "Nick change collision from %s to %s(%s <- %s)(both killed)",
 		 source_p->name, target_p->name, target_p->from->name,
 		 client_p->name);
@@ -921,18 +929,18 @@ perform_nick_collides(struct Client *source_p, struct Client *client_p,
 
       ServerStats->is_kill++;
       /* If we got the message from a LL, ensure it gets the kill */
-      if(ServerInfo.hub && IsCapable(client_p,CAP_LL))
+      if (ServerInfo.hub && IsCapable(client_p,CAP_LL))
         add_lazylinkclient(client_p, target_p);
 
       kill_client_ll_serv_butone(NULL, target_p,
                                  "%s (Nick change collision)",
 				 me.name);
 
-      target_p->flags |= FLAGS_KILLED;
-      exit_client(NULL, target_p, &me, "Nick collision(new)");
-      source_p->flags |= FLAGS_KILLED;
-      exit_client(client_p, source_p, &me, "Nick collision(old)");
-      return 0;
+      SetKilled(target_p);
+      exit_client(target_p, &me, "Nick collision (new)");
+      SetKilled(source_p);
+      exit_client(source_p, &me, "Nick collision (old)");
+      return;
     }
     else
     {
@@ -942,13 +950,13 @@ perform_nick_collides(struct Client *source_p, struct Client *client_p,
       if ((sameuser && newts < target_p->tsinfo) ||
           (!sameuser && newts > target_p->tsinfo))
       {
-        if(sameuser)
-	  sendto_gnotice_flags(FLAGS_ALL, L_OPER, me.name, &me, NULL,
+        if (sameuser)
+	  sendto_gnotice_flags(UMODE_ALL, L_ALL, me.name, &me, NULL,
  	       "Nick change collision from %s to %s(%s <- %s)(older killed)",
 	       source_p->name, target_p->name, target_p->from->name,
 	       client_p->name);
         else
-	  sendto_gnotice_flags(FLAGS_ALL, L_OPER, me.name, &me, NULL,
+	  sendto_gnotice_flags(UMODE_ALL, L_ALL, me.name, &me, NULL,
 	       "Nick change collision from %s to %s(%s <- %s)(newer killed)",
 	       source_p->name, target_p->name, target_p->from->name,
 	       client_p->name);
@@ -960,23 +968,23 @@ perform_nick_collides(struct Client *source_p, struct Client *client_p,
 	                           "%s (Nick change collision)",
 				   me.name);
 
-        source_p->flags |= FLAGS_KILLED;
+        SetKilled(source_p);
 	
-	if(sameuser)
-	  exit_client(client_p, source_p, &me, "Nick collision(old)");
+	if (sameuser)
+	  exit_client(source_p, &me, "Nick collision (old)");
 	else
-	  exit_client(client_p, source_p, &me, "Nick collision(new)");
-	return 0;
+	  exit_client(source_p, &me, "Nick collision (new)");
+	return;
      }
      else
      {
-       if(sameuser)
-         sendto_gnotice_flags(FLAGS_ALL, L_OPER, me.name, &me, NULL,
+       if (sameuser)
+         sendto_gnotice_flags(UMODE_ALL, L_ALL, me.name, &me, NULL,
 	                      "Nick collision on %s(%s <- %s)(older killed)",
 			      target_p->name, target_p->from->name,
 			      client_p->name);
        else
-         sendto_gnotice_flags(FLAGS_ALL, L_OPER, me.name, &me, NULL,
+         sendto_gnotice_flags(UMODE_ALL, L_ALL, me.name, &me, NULL,
 	                      "Nick collision on %s(%s <- %s)(newer killed)",
 			      target_p->name, target_p->from->name,
 			      client_p->name);
@@ -989,24 +997,13 @@ perform_nick_collides(struct Client *source_p, struct Client *client_p,
        sendto_one(target_p, form_str(ERR_NICKCOLLISION),
                   me.name, target_p->name, target_p->name);
 
-       target_p->flags |= FLAGS_KILLED;
-       (void)exit_client(client_p, target_p, &me, "Nick collision");
+       SetKilled(target_p);
+       exit_client(target_p, &me, "Nick collision");
      }
    }
 
-   /*
-   if(HasID(source_p))
-     client_from_server(client_p,source_p,parc,parv,newts,nick);
-   else
+  /* we should only ever call nick_from_server() here, as
+   * this is a client changing nick, not a new client
    */
-   
-   /* we should only ever call nick_from_server() here, as
-    * this is a client changing nick, not a new client
-    */
-   nick_from_server(client_p,source_p,parc,parv,newts,nick);
-
-  return 0;
+  nick_from_server(client_p, source_p, parc, parv, newts, nick, gecos);
 }
-
-	    
-
