@@ -21,7 +21,7 @@
 
 /*! \file channel.c
  * \brief Responsible for managing channels, members, bans and topics
- * \version $Id: channel.c 801 2007-02-15 22:45:29Z stu $
+ * \version $Id: channel.c 859 2007-04-09 17:38:36Z stu $
  */
 
 #include "stdinc.h"
@@ -705,10 +705,8 @@ can_join(struct Client *source_p, struct Channel *chptr, const char *key)
   if (SSLonlyChannel(chptr))
   {
 #ifdef HAVE_LIBCRYPTO
-    if (MyClient(source_p)) {
-      if (source_p->localClient->fd.ssl)
+    if (MyClient(source_p) && !(source_p->localClient->fd.ssl))
         return (ERR_SSLONLYCHAN);
-    }
 #else
     return (ERR_SSLONLYCHAN);  /* deny everyone on a non SSL-enabled server */
 #endif
