@@ -19,7 +19,7 @@
  *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
  *  USA
  *
- *  $Id: client.c 836 2007-02-19 21:47:40Z stu $
+ *  $Id: client.c 848 2007-02-22 19:31:31Z stu $
  */
 
 #include "stdinc.h"
@@ -999,10 +999,14 @@ exit_client(struct Client *source_p, struct Client *from, const char *comment)
                            source_p->name, source_p->username, source_p->host, comment,
                            ConfigFileEntry.hide_spoof_ips && IsIPSpoof(source_p) ?
                            "255.255.255.255" : source_p->sockhost);
-      sendto_realops_flags(UMODE_CCONN_FULL, L_ALL, "CLIEXIT: %s %s %s %s %s",
-                           source_p->name, source_p->username, source_p->host, comment,
+      sendto_realops_flags(UMODE_CCONN_FULL, L_ALL, "CLIEXIT: %s %s %s %s 0 %s",
+                           source_p->name,
+			   source_p->username,
+			   source_p->host,
+
                            ConfigFileEntry.hide_spoof_ips && IsIPSpoof(source_p) ?
-                           "255.255.255.255" : source_p->sockhost);
+                           "255.255.255.255" : source_p->sockhost,
+			   comment);
     }
 
     /* As soon as a client is known to be a server of some sort
