@@ -19,7 +19,7 @@
  *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
  *  USA
  *
- *  $Id: hostmask.c 803 2007-02-16 09:31:25Z stu $
+ *  $Id: hostmask.c 883 2007-04-16 13:43:29Z stu $
  */
 
 #include "stdinc.h"
@@ -31,6 +31,8 @@
 #include "numeric.h"
 #include "send.h"
 #include "irc_string.h"
+#include "sprintf_irc.h"
+#include "s_misc.h"
 
 #ifdef IPV6
 static int try_parse_v6_netmask(const char *, struct irc_ssaddr *, int *);
@@ -897,6 +899,7 @@ report_Klines(struct Client *client_p, int tkline)
   struct AccessItem *aconf = NULL;
   int i;
   const char *p = NULL;
+  char buf[IRCD_BUFSIZE+1] = {'\0'};
 
   if (tkline)
     p = "k";
@@ -913,14 +916,20 @@ report_Klines(struct Client *client_p, int tkline)
             (!tkline && ((aconf = arec->aconf)->flags & CONF_FLAGS_TEMPORARY)))
           continue;
 
-	if (IsOper(client_p))
-	  sendto_one(client_p, form_str(RPL_STATSKLINE), me.name,
-                     client_p->name, p, aconf->host, aconf->user,
-		     aconf->reason, aconf->oper_reason ? aconf->oper_reason : "");
-	else
+        if (IsOper(client_p))
+        {
+          ircsprintf(buf, "%s Expires (%s)", 
+              aconf->oper_reason ? aconf->oper_reason : "", 
+              smalldate(aconf->hold));
+
           sendto_one(client_p, form_str(RPL_STATSKLINE), me.name,
-                     client_p->name, p, aconf->host, aconf->user,
-		     aconf->reason, "");
+              client_p->name, p, aconf->host, aconf->user,
+              aconf->reason, buf);
+        }
+        else
+          sendto_one(client_p, form_str(RPL_STATSKLINE), me.name,
+              client_p->name, p, aconf->host, aconf->user,
+              aconf->reason, "");
       }
     }
   }
