@@ -19,7 +19,7 @@
  *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
  *  USA
  *
- *  $Id: s_user.c 836 2007-02-19 21:47:40Z stu $
+ *  $Id: s_user.c 856 2007-04-09 16:59:04Z stu $
  */
 
 #include "stdinc.h"
@@ -903,7 +903,13 @@ change_simple_umode(va_list args)
   if (what == MODE_ADD)
   {
     if(flag == UMODE_GOD && !IsGod(source_p) && MyConnect(source_p))
+    {
+      char tmp[IRCD_BUFSIZE];
+      ircsprintf(tmp, "%s is setting God mode", source_p->name);
+      sendto_gnotice_flags(UMODE_SERVNOTICE, L_ALL, me.name, &me, NULL, tmp);
+      oftc_log(tmp);
       source_p->umodestime = CurrentTime;
+    }
     source_p->umodes |= flag;
   }
   else
